## Slider 

Sliders allow users to make selections from a range of values.


### Basic Slider

```Python
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout
class App:
    @mark_create_layout
    def my_layout(self):
        return mui.VBox([
            # begin, end, step
            mui.Slider(0, 100, 1),
            mui.Slider(0, 0),
            mui.Slider(0, 100, 1).prop(size="small"),
        ]).prop(width="100%")

```

### Range Slider


```Python
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout
class App:
    @mark_create_layout
    def my_layout(self):
        md = mui.Markdown()
        return mui.VBox([
            # begin, end, step
            mui.RangeSlider(0, 100, 1, init_value=(20, 30), callback=lambda x: md.write(f"RangeSlider: :green[{x}]")),
            md,
        ]).prop(width="100%")

```

### Vertical Slider


```Python
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout
class App:
    @mark_create_layout
    def my_layout(self):
        return mui.VBox([
            # begin, end, step
            mui.Slider(0, 100, 1).prop(vertical=True, flex=1),
        ]).prop(height="100%")

```

### Slider with text input

Double click the text to edit the value.

```Python
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout
class App:
    @mark_create_layout
    def my_layout(self):
        return mui.VBox([
            # begin, end, step
            mui.Slider(0, 100, 1).prop(valueInput=True),
        ]).prop(width="100%")

```


### Blender-style Numeric Slider

mui slider isn't suitable for numeric control. so we add a blender-style numeric slider.

```Python
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout
class App:
    @mark_create_layout
    def my_layout(self):
        return mui.VBox([
            mui.Markdown("### Default slider"),
            # begin, end, step
            mui.BlenderSlider(0, 100, 1),
            mui.Markdown("### Infinite slider"),
            mui.Markdown("\nYou must specify step and init_value if you want to use infinite slider (no begin or end)."),
            mui.BlenderSlider(step=1, init_value=10),
        ]).prop(width="100%")

```
