## Video Player

### Basic Usage

```Python
from tensorpc.flow import (marker, mui)

class App:

    @marker.mark_create_layout
    def my_layout(self):
        video_src = "https://files.vidstack.io/sprite-fight/240p.mp4"
        thumbnails_src = "https://files.vidstack.io/sprite-fight/thumbnails.vtt"
        return mui.HBox([
            mui.VideoPlayer(src=video_src).prop(title="Demo", thumbnails=thumbnails_src)
        ]).prop(width="100%")

```


### Host Local Videos

```Python
from tensorpc.flow import (marker, mui, appctx)
import aiohttp
async def download_file(url: str, chunk_size: int = 2**16):
    chunks = []
    async with aiohttp.ClientSession() as sess:
        async with sess.get(url) as response:
            assert response.status == 200
            while True:
                chunk = await response.content.read(chunk_size)
                if not chunk:
                    break
                chunks.append(chunk)
    return b"".join(chunks)

class App:

    @marker.mark_create_layout
    def my_layout(self):
        video_src = "tensorpc://demo-video.mp4"
        thumbnails_src = "https://files.vidstack.io/sprite-fight/thumbnails.vtt"

        self._video_binary = None
        appctx.get_app().add_file_resource("demo-video.mp4",
                                           self._video_demo_resource)

        return mui.HBox([
            mui.VideoPlayer(src=video_src).prop(title="Demo", thumbnails=thumbnails_src)
        ]).prop(width="100%")

    async def _video_demo_resource(self, req):
        # we need to download whole video to memory, so we only use 240p
        url = "https://files.vidstack.io/sprite-fight/240p.mp4"
        if self._video_binary is None:
            self._video_binary = await download_file(url)
        print("video downloaded", len(self._video_binary))
        return mui.FileResource(name="demo-video.mp4",
                                content=self._video_binary)


```

