## Flow

You can create a simple app with the following code:

```Python 
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout, flowui

flow_css = {
    ".react-flow__node-circle": {
        "border-radius": "50%",
        "height": "100px",
        "width": "100px",
        "font-family": "monospace",
        "text-align": "center",
    }
}

class App:
    
    class Config(plus.AppInMemory.Config):
        is_horizontal = False
        height = "800px"

    @mark_create_layout
    def my_layout(self):
        tools_content = mui.Typography("🚀")
        nodes = [
            flowui.Node(id="1-1", type="input", data=flowui.NodeData(label="Input Node"), position=flowui.XYPosition(150, 0)),
            flowui.Node(id="1-2", type="default", data=flowui.NodeData(label="Default Node"), position=flowui.XYPosition(0, 100)),
            flowui.Node(id="1-3", type="output", data=flowui.NodeData(label="Output Node"), position=flowui.XYPosition(300, 100)),
            flowui.Node(id="2-1", type="group", position=flowui.XYPosition(-170, 250), style={
                "width": 380,
                "height": 180,
                "backgroundColor": 'rgba(208, 192, 247, 0.2)',
            }),
            flowui.Node(id="2-2", type="tools", data=flowui.NodeData(component=mui.Fragment([
                flowui.NodeToolbar([
                    mui.ButtonGroup([
                        mui.Button("🚀", lambda: tools_content.write("🚀")).prop(size="small"),
                        mui.Button("🔥", lambda: tools_content.write("🔥")).prop(size="small"),
                        mui.Button("✨", lambda: tools_content.write("✨")).prop(size="small"),
                    ])
                ]).prop(isVisible=True),
                flowui.Handle(type="target", position="left"),
                flowui.Handle(type="source", position="right"),
                tools_content,
            ])), position=flowui.XYPosition(50, 50), style={
                "width": 80,
                "height": 80,
                "backgroundColor": 'rgba(208, 192, 247)',
            }, parentId="2-1", extent="parent"),
            flowui.Node(id="2-3", type="resizer", data=flowui.NodeData(component=mui.Fragment([
                flowui.NodeResizer().prop(minWidth=50, minHeight=50),
                flowui.Handle(type="target", position="left"),
                mui.Typography("resizable node").prop(variant="caption"),
                mui.HBox([
                    flowui.Handle(type="source", position="bottom").prop(style={"position": 'relative', "left": 0, "transform": 'none' }, id="a"),
                    flowui.Handle(type="source", position="bottom").prop(style={"position": 'relative', "left": 0, "transform": 'none' }, id="b"),
                ]).prop(justifyContent="space-evenly", width="100%", bottom=0, position="absolute"),

            ])), position=flowui.XYPosition(250, 50), style={
                "width": 80,
                "height": 80,
                "backgroundColor": 'rgba(208, 192, 247)',
            }, parentId="2-1", extent="parent"),
            flowui.Node(id="3-2", type="textinput", data=flowui.NodeData(component=mui.VBox([
                mui.VBox([
                    mui.VBox([
                        mui.HBox([
                            mui.Typography("drag me!").prop(variant="body1")
                        ]).prop(className="custom-drag-handle"),
                        mui.Divider(),
                        mui.Typography("node width").prop(variant="body1"),
                        mui.BlenderSlider(50, 200, callback=self._update_node_width, init_value=80).prop(showControlButton=True),
                        # mui.TextField("width", self._update_node_width, init="80").prop(type="number"),
                        mui.Typography("node height").prop(variant="body1"),
                        mui.TextField("height", self._update_node_height, init="80").prop(type="number"),
                    ]).prop(className="inner"),
                ]).prop(className="wrapper gradient"),
                flowui.Handle(type="target", position="top"),

            ])), position=flowui.XYPosition(150, 650), dragHandle=".custom-drag-handle"),
            flowui.Node(id="3-1", type="circle", data=flowui.NodeData(component=mui.Fragment([
                mui.HBox([
                    mui.HBox([
                        mui.Typography("circle").prop(variant="body1")
                    ]).prop(className="inner"),
                ]).prop(className="wrapper gradient"),
                flowui.Handle(type="target", position="left"),

            ])), position=flowui.XYPosition(350, 500)),

        ]
        edges = [
            flowui.Edge("e1-2", source="1-1", target="1-2", label="edge", type="smoothstep"),
            flowui.Edge("e1-3", source="1-1", target="1-3", label="animated edge", animated=True),
            flowui.Edge("e2-2", source="1-2", target="2-2", type="smoothstep", markerEnd=flowui.EdgeMarker(type="arrowclosed")),
            flowui.Edge("e2-3", source="2-2", target="2-3", type="smoothstep", markerEnd=flowui.EdgeMarker(type="arrowclosed")),
            flowui.Edge("e3-3", source="2-3", sourceHandle="a", target="3-2", animated=True, style={
                "stroke": 'rgb(158, 118, 255)', "strokeWidth": 2
            }),
            flowui.Edge("e3-4", source="2-3", sourceHandle="b", target="3-1", style={
                "strokeWidth": 2
            }),

        ]
        self.graph = flowui.Flow(nodes, edges, [
                flowui.MiniMap(),
                flowui.Controls(),
                flowui.Background()
            ]).prop(className="overview", nodeTypeMap={
                "tools": "app",
                "resizer": "app",
                "circle": "app",
                "textinput": "app",
            })
        return mui.HBox([
            self.graph,
        ]).prop(width="100%", height="100%", overflow="hidden").update_raw_props(flow_css)

    async def _update_node_width(self, value: int):
        int_value = int(value)
        await self.graph.update_node_style("2-3", {
            "width": int_value
        })

    async def _update_node_height(self, value: str):
        int_value = int(value)
        await self.graph.update_node_style("2-3", {
            "height": int_value
        })

```

