## Symbolic Flow

You can create a simple app with the following code:

```Python 
from tensorpc.flow import mui, three, plus, appctx, mark_create_layout, flowui, mark_did_mount
class App:
    
    class Config(plus.AppInMemory.Config):
        is_horizontal = False
        height = "800px"

    @mark_create_layout
    def my_layout(self):
        graph = flowui.Flow([], [], [
            flowui.MiniMap(),
            flowui.Controls(),
            flowui.Background()
        ])
        self.graph = graph
        return mui.HBox([
            graph.prop(defaultLayoutSize=(150, 50)),
        ]).prop(width="100%", height="100%", overflow="hidden")

    @mark_did_mount
    async def _on_mount(self):
        symbolic_flow_bd = flowui.SymbolicFlowBuilder()
        a, a_node = symbolic_flow_bd.create_input("a")
        b, b_node = symbolic_flow_bd.create_input("b")

        a_node.styles = {
            "backgroundColor": "red",
        }

        add_op = symbolic_flow_bd.create_op_node("Add", ["a", "b"], ["c"])
        c_list = symbolic_flow_bd.call_op_node(add_op, {"a": a, "b": b})
        graph_res = symbolic_flow_bd.build_detached_flow(c_list)
        nodes, edges, node_type_map = graph_res.nodes, graph_res.edges, graph_res.node_type_map
        await self.graph.set_flow_and_do_dagre_layout(nodes, edges)
```

